local lg = love.graphics
local enet = require "enet"
local updateRate = 0.1

-- so far, both "*" and "0.0.0.0" seem to work (same PC or WLAN)
-- and ofc "localhost" when running on the same PC
local SERVER_IP = require "server_ip"
local SERVER_PORT = 16666
local MAX_PEER_COUNT = 4

----------------------------------------------------------------------------------------------------

function love.load()
	
end

----------------------------------------------------------------------------------------------------

function love.update( dt )
	if server then
		server:tick( dt )
	elseif client then
		client:tick( dt )
	end
end

----------------------------------------------------------------------------------------------------

function love.draw()
	
	if server then
		lg.print( "SERVER", 10, 5 )
		
		for i = 1, MAX_PEER_COUNT do
			local peer = server.host:get_peer( i )
			if peer then
				lg.print( string.format( "%d - %s [%d]", i, peer:state(), peer:last_round_trip_time() ), 10, 5 + 10 * i)
			end
		end
		
	elseif client then
		lg.print( "CLIENT", 10, 5 )
		lg.print( string.format( "%s [%d]", client.server:state(), client.server:last_round_trip_time() ), 10, 15 )
		
	end
	
end

----------------------------------------------------------------------------------------------------

local Server = {}

function Server:create()
	local server = {}
	
	server.time = 0
	server.updateTick = 0
	
	server.host = enet.host_create( string.format( "%s:%d", SERVER_IP, SERVER_PORT) , MAX_PEER_COUNT )
	print( "Create server at", server.host:get_socket_address() )
	
	function server:tick( dt )
		
		self.time = self.time + dt
		
		-- update rate
		self.updateTick = self.updateTick + dt
		if self.updateTick > updateRate then
			self.updateTick = 0
			
			self:listen()
		end
		
	end
	
	
	function server:listen()
		local event
		repeat
			event = self.host:service()
			if event then
				if event.type == "receive" then
					print( "received:", event.data )
					print( "from", event.peer )
					event.peer:send( "((answer))" )
					
				elseif event.type == "connect" then
					print( "Client connected", event.peer )
					event.peer:send( "hallo, client" )
					
					print( "peer:", event.peer:index() .. "/" .. self.host:peer_count() )
				elseif event.type == "disconnect" then
					print( "Client disconnected", event.peer )
					
				end
			end
			event = self.host:service()
		until not event
	end
	
	
	return server
end

----------------------------------------------------------------------------------------------------

local Client = {}

function Client:create()
	local client = {}
	
	client.time = 0
	client.updateTick = 0
	
	client.host = enet.host_create()
	client.server = client.host:connect( string.format( "%s:%d", SERVER_IP, SERVER_PORT) )
	print( "Create client at", client.host:get_socket_address() )
	
	function client:tick( dt )
		
		self.time = self.time + dt
		
		-- update rate
		self.updateTick = self.updateTick + dt
		if self.updateTick > updateRate then
			self.updateTick = 0
			
			self:listen()
		end
		
	end
	
	
	function client:listen()
		
		local event
		repeat
			event = self.host:service()
			if event then
				if event.type == "receive" then
					print( "received:", event.data )
					print( "from", event.peer )
				elseif event.type == "connect" then
					print( "Client connected", event.peer )
				elseif event.type == "disconnect" then
					print( "Client disconnected", event.peer )
				end
			event = self.host:service()
			end
		until not event
		
	end
	
	return client
end

----------------------------------------------------------------------------------------------------

function love.keypressed( key, scancode, isrepeat )
	
	if not server and not client then
		if key == "s" then
			server = Server:create()
		elseif key == "c" then
			client = Client:create()
		end
	end
	
	if key == "escape" then
		if client then
			client.server:disconnect()
			client.host:flush()
			client.host:destroy()
			client = nil
			
		elseif server then
			server.host:destroy()
			server = nil
			
		end
		
	elseif key == "m" then
		if client then
			-- client.host:service()
			for i = 1,5 do
				client.server:send( "guten tag " .. i )
			end
			
		elseif server then
			server.host:broadcast( "hallo an alle" )
			
		end
		
	end
	
end
